/*
 * Designed and developed by 2022 Ji Sungbin.
 *
 * Licensed under the MIT.
 * Please see full license: https://github.com/duckie-team/ApiLibrary/blob/trunk/LICENSE
 */

@file:Suppress("SameParameterValue")

package land.sungbin.apilibrary.convention

import org.gradle.api.artifacts.dsl.DependencyHandler as DependencyScope
import com.android.build.api.dsl.CommonExtension
import org.gradle.api.Action
import org.gradle.api.NamedDomainObjectContainer
import org.gradle.api.Project
import org.gradle.api.artifacts.VersionCatalog
import org.gradle.api.artifacts.VersionCatalogsExtension
import org.gradle.api.plugins.ExtensionAware
import org.gradle.kotlin.dsl.NamedDomainObjectContainerScope
import org.gradle.kotlin.dsl.getByType
import org.jetbrains.kotlin.gradle.dsl.KotlinJvmOptions

/**
 * 그레이들에서 좀 더 쉽게 사용하기 위한 확장 함수들을 정의합니다.
 */
private const val Api = "api"
private const val CompileOnly = "compileOnly"
private const val Implementation = "implementation"
private const val DebugImplementation = "debugImplementation"
private const val TestRuntimeOnly = "testRuntimeOnly"
private const val TestImplementation = "testImplementation"
private const val AndroidTestImplementation = "androidTestImplementation"

internal fun Project.applyPlugins(vararg plugins: String) {
    plugins.forEach { plugin ->
        pluginManager.apply(plugin)
    }
}

internal val Project.libs
    get() = extensions.getByType<VersionCatalogsExtension>().named("libs")

@Suppress("nothing_to_inline")
internal inline operator fun <T : Any, C : NamedDomainObjectContainer<T>> C.invoke(
    configuration: Action<NamedDomainObjectContainerScope<T>>,
) = apply {
    configuration.execute(
        NamedDomainObjectContainerScope.of(
            container = this,
        )
    )
}

internal fun CommonExtension<*, *, *, *>.kotlinOptions(block: KotlinJvmOptions.() -> Unit) {
    (this as ExtensionAware).extensions.configure("kotlinOptions", block)
}

internal fun DependencyScope.apis(
    vararg paths: Any,
) {
    delegate(
        method = Api,
        paths = paths,
    )
}

internal fun DependencyScope.compileOnlys(
    vararg paths: Any,
) {
    delegate(
        method = CompileOnly,
        paths = paths,
    )
}

internal fun DependencyScope.implementations(
    vararg paths: Any,
) {
    delegate(
        method = Implementation,
        paths = paths,
    )
}

internal fun DependencyScope.testImplementations(
    vararg paths: Any,
) {
    delegate(
        method = TestImplementation,
        paths = paths,
    )
}

internal fun DependencyScope.androidTestImplementations(
    vararg paths: Any,
) {
    delegate(
        method = AndroidTestImplementation,
        paths = paths,
    )
}

internal fun DependencyScope.debugImplementations(
    vararg paths: Any,
) {
    delegate(
        method = DebugImplementation,
        paths = paths,
    )
}

internal fun DependencyScope.setupJunit(
    core: Any,
    engine: Any,
) {
    delegate(
        method = TestImplementation,
        paths = arrayOf(core),
    )
    delegate(
        method = TestRuntimeOnly,
        paths = arrayOf(engine),
    )
}

internal fun DependencyScope.setupLint(
    core: Any,
    test: Any,
) {
    delegate(
        method = CompileOnly,
        paths = arrayOf(core),
    )
    delegate(
        method = TestImplementation,
        paths = arrayOf(test),
    )
}

internal fun DependencyScope.setupComposeBom(
    libs: VersionCatalog,
) {
    val bom = platform(libs.findLibrary("compose-bom").get())
    delegate(
        method = Implementation,
        paths = arrayOf(bom),
    )
    delegate(
        method = DebugImplementation,
        paths = arrayOf(bom),
    )
    delegate(
        method = AndroidTestImplementation,
        paths = arrayOf(bom),
    )
}

internal fun DependencyScope.setupComposeUi(
    libs: VersionCatalog,
) {
    implementations(
        libs.findLibrary("compose-material3").get(),
        libs.findLibrary("compose-activity").get(),
    )
}

private fun DependencyScope.delegate(
    method: String,
    vararg paths: Any,
) {
    paths.forEach { path ->
        add(method, path)
    }
}
